--
-- @author: Ziuta (DC: ja_pizgam#0042)
-- @date: 11.11.2018
-- @version: 1.0.0.0
-- @edit: 08.04.2019 - v1.0.0.0 convert to FS19
-- @edit2: 19.11.2019 - v1.0.0.1 fix FPS drop
-- @edit3: 16.06.2022 - v1.0.0.0 convert to FS22: clean, fix and improve code
-- @edit4: 04.03.2023 - v1.1.0.0 code rewriting, better adaptation to tools
-- @edit5: 11.03.2023 - v1.2.0.0 code fixes (thx Wopster), better adaptation for small tools, support for headers/adapters

local isOnlyUpDown = false -- true - only up-down rocking ***** false - right-left and up-down rocking
local cutterAlso = true -- true - activates rocking herers/adapters ***** false - deactivates rocking herers/adapters

local function loadInputAttacherJoint(self, superFunc, xmlFile, key, inputAttacherJoint, index)
    superFunc(self, xmlFile, key, inputAttacherJoint, index)
    local specSprayer = self.spec_sprayer
    local specMotorized = self.spec_motorized
    local isImplement = inputAttacherJoint.jointType == AttacherJoints.JOINTTYPE_IMPLEMENT or false
    local isCutter = inputAttacherJoint.jointType == AttacherJoints.JOINTTYPE_CUTTER or inputAttacherJoint.jointType == AttacherJoints.JOINTTYPE_CUTTERHARVESTER or false

    if cutterAlso and isCutter then
        local data = {
            weight = self.defaultMass,
            limitSpringZ = 700,
            limitDampingZ = 5}
        if data.weight > 2 then
            data.limitSpringZ = data.limitSpringZ + 700
            data.limitDampingZ = data.limitDampingZ + 35
        end
        inputAttacherJoint.rotLimitSpring = {0,0,data.limitSpringZ}
        inputAttacherJoint.rotLimitDamping = {0,0,data.limitDampingZ}
        return true
    end

    if isImplement then
        local _, rotY, _ = getRotation(inputAttacherJoint.node)
        local isFrontAttacher = math.floor(math.deg(rotY)) == -91 or false -- check it is front or back attacher
        local isTPA = inputAttacherJoint.topReferenceNode ~= nil or false -- check it is three or two point attacher
        local isFertilizerSprayer = false -- loaded spreader must have a stronger spring
        if specSprayer ~= nil then
            isFertilizerSprayer = specSprayer.isFertilizerSprayer
        end
        local data = {
            weight = self.defaultMass,
            limitSpringYFront = 400,
            limitSpringYBack = 50,
            limitSpringZ = 50,
            limitDampingY = 5,
            limitDampingZ = 0 }
        if isFrontAttacher then
            if data.weight > 1 then
                data.limitSpringYFront = data.limitSpringYFront + 500
                data.limitSpringZ = data.limitSpringZ + 550
                if isFertilizerSprayer then
                    data.limitSpringYFront = data.limitSpringYFront + 500
                    data.limitSpringZ = data.limitSpringZ + 350
                end
            end
            inputAttacherJoint.transLimitSpring = {0,data.limitSpringYFront,0}
            inputAttacherJoint.transLimitDamping = {0,data.limitDampingY,0}
        else
            if data.weight > 0.5 then
                data.limitSpringYBack = data.limitSpringYBack + 950
                data.limitSpringZ = data.limitSpringZ + 550
                if isFertilizerSprayer then
                    data.limitSpringYBack = data.limitSpringYBack + 1500
                    data.limitSpringZ = data.limitSpringZ + 350
                end
            end
            if isOnlyUpDown then
                data.limitSpringZ = 0
            else
                if isTPA then
                    data.limitDampingZ = 1
                    if data.weight > 0.5 then
                        data.limitDampingZ = data.limitDampingZ + 4
                    end
                else
                    data.limitDampingZ = data.limitDampingZ + 30
                end
            end
            inputAttacherJoint.transLimitSpring = {0,data.limitSpringYBack,data.limitSpringZ}
            inputAttacherJoint.transLimitDamping = {0,data.limitDampingY,data.limitDampingZ}
        end        
        return true
    end
    return superFunc(self, xmlFile, key, inputAttacherJoint, index)
end
Attachable.loadInputAttacherJoint = Utils.overwrittenFunction(Attachable.loadInputAttacherJoint, loadInputAttacherJoint)

if not isOnlyUpDown then    
    local function updateAttacherJointGraphics(self, implement, dt, forceUpdate)
        if implement.object ~= nil then
            local spec = self.spec_attacherJoints
            local jointDesc = spec.attacherJoints[implement.jointDescIndex]

			if jointDesc.bottomArm ~= nil then
                if jointDesc.bottomArm.rotationNode ~= nil and jointDesc.bottomArm.translationNode ~= nil then
					local attacherJoint = implement.object:getActiveInputAttacherJoint()

			        -- ▼▼▼▼▼▼▼▼▼ Wopster ▼▼▼▼▼▼▼▼▼
					if jointDesc.bottomArm.realTranslationNode == nil then
						jointDesc.bottomArm.realTranslationNode = createTransformGroup("realTranslationNode")
						-- Wopster: we only need to tell the scenegraph to link it once.
						link(jointDesc.bottomArm.rotationNode, jointDesc.bottomArm.realTranslationNode)
						link(jointDesc.bottomArm.realTranslationNode, jointDesc.bottomArm.translationNode)
					end
                    -- ▲▲▲▲▲▲▲▲▲ Wopster ▲▲▲▲▲▲▲▲▲
                    local x, _, _ = worldToLocal(jointDesc.bottomArm.rotationNode, getWorldTranslation(attacherJoint.node))
                    setTranslation(jointDesc.bottomArm.realTranslationNode, x, 0, 0)
                end
            end
        end
    end

    AttacherJoints.updateAttacherJointGraphics = Utils.appendedFunction(AttacherJoints.updateAttacherJointGraphics, updateAttacherJointGraphics)
end